// Copyright 2016 Orus.io. All rights reserved.

package control

import (
	"gopkg.in/yaml.v2"

	"bitbucket.org/orus-io/go-xbus/api"
	"bitbucket.org/orus-io/xbus/graph"
)

func (s *Service) handleGraphQuery(subject, reply string, req api.ControlGraphQueryReq) {
	var rep api.ControlGraphQueryRep

	s.TaskAdd()
	defer s.SendReply(reply, &rep)

	graphs := s.o.GraphManager.Query(req.Active)
	for _, graph := range graphs {
		b, err := yaml.Marshal(graph)
		if err != nil {
			rep.Response = api.ServerError(err)
			return
		}
		rep.GraphList = append(rep.GraphList, string(b))
	}
	rep.Status = api.StatusOK
}

func (s *Service) handleGraphPush(subject, reply string, req api.ControlGraphPushReq) {
	var rep api.ControlGraphPushRep

	s.TaskAdd()
	defer s.SendReply(reply, &rep)

	var graph graph.Graph
	if err := yaml.Unmarshal([]byte(req.Graph), &graph); err != nil {
		rep.Response = api.ClientError("Error decoding graph: %s", err)
		return
	}

	for _, e := range graph.Validate() {
		rep.ValidationMessages = append(rep.ValidationMessages, e.Error())
	}

	for _, e := range graph.ValidateActors(s.o.ActorStorage) {
		rep.ValidationMessages = append(rep.ValidationMessages, e.Error())
	}

	graph, err := s.o.GraphManager.Save(graph)
	if err != nil {
		rep.Response = api.ClientError("Save graph error: %s", err)
		return
	}

	b, err := yaml.Marshal(graph)
	if err != nil {
		rep.Response = api.ServerError(err)
		return
	}
	rep.Status = api.StatusOK
	rep.Graph = string(b)
}
