// Copyright 2016 Orus.io. All rights reserved.

package control

import (
	"crypto/x509"

	"bitbucket.org/orus-io/go-xbus/api"
	"bitbucket.org/orus-io/xbus/graph"
	"bitbucket.org/orus-io/xbus/services"
	"bitbucket.org/orus-io/xbus/storage"
)

// Options for the Service
type Options struct {
	services.Options

	ServerCert *x509.Certificate
	ServerKey  interface{}

	GraphManager   graph.Manager
	ProcessStorage storage.ProcessStorage
	AccountStorage storage.AccountStorage
	ActorStorage   storage.ActorStorage
}

// NewService instanciate a Service
func NewService(o Options) *Service {
	return &Service{
		o:           o,
		BaseService: services.NewBaseService("control", o.Options),
	}
}

// Service provides an api for administrators
type Service struct {
	services.BaseService

	o Options
}

// Startup starts the service
func (s *Service) Startup() error {
	if err := s.SubscribeAll(
		services.Subscribe(api.ControlActorsPrefix+"*", s.handleActorCRUD),
		services.Subscribe(api.ControlGraphQuery, s.handleGraphQuery),
		services.Subscribe(api.ControlGraphPush, s.handleGraphPush),
		services.Subscribe(api.ControlProcessQuery, s.handleProcessQuery),
		services.Subscribe(api.ControlProcessControl, s.handleProcessControl),
		services.Subscribe(api.ControlAccountList, s.handleAccountList),
		services.Subscribe(api.ControlAccountUpdate, s.handleAccountUpdate),
	); err != nil {
		s.Shutdown()
		return err
	}
	return nil
}
