// Copyright 2016 Orus.io. All rights reserved.

package storage

import (
	"github.com/m4rw3r/uuid"

	"bitbucket.org/orus-io/go-xbus/api"
)

// ActorEventHandler is the type for callbacks
type ActorEventHandler func(api.Actor)

// NoopActorEventHandler is an ActorEventHandler that does nothing
func NoopActorEventHandler(api.Actor) {}

// ActorStorage is in charge of storing the actor definitions
// When multiple instances are running, CRUD operations be reflected from one
// to the others
type ActorStorage interface {
	Get(uuid.UUID) (api.Actor, bool)
	GetByName(name string) (api.Actor, bool)
	List() []api.Actor
	ListAccount(accountID uuid.UUID) []api.Actor
	ListKind(actorKind api.ActorKind) []api.Actor
	ListRole(actorKind api.ActorKind, role string) []api.Actor

	Create(api.Actor) (api.Actor, error)
	Update(api.Actor) (api.Actor, error)

	MarkUnresponsive(uuid.UUID) error
	JustSeen(uuid.UUID) error

	SetEventHandlers(onActorUp, onActorDown ActorEventHandler)
}
