// Copyright 2017 Orus.io. All rights reserved.

package uuid

import (
	"testing"

	"github.com/stretchr/testify/assert"
)

func TestNullUUID(t *testing.T) {
	s := "9103294c-d281-11e6-9d82-93ae60dc4ded"

	assert.Equal(t,
		"",
		NullUUID{}.String())

	assert.Equal(t, NullUUID{}, MustFromString(""))
	assert.Equal(t, NullUUID{}, MustFromString("null"))
	assert.Equal(t, NullUUID{}, MustFromBytes([]byte("")))
	assert.Equal(t, NullUUID{}, MustFromBytes([]byte("null")))

	assert.Equal(t, s, MustFromString(s).String())

	assert.Panics(t, func() { MustFromString("invalid") })
	assert.Panics(t, func() { MustFromBytes([]byte("invalid")) })

	b, err := MustFromString(s).MarshalText()
	assert.Nil(t, err)
	assert.Equal(t, string(b), s)

	var u NullUUID
	assert.Nil(t, u.UnmarshalText([]byte(s)))
	assert.Equal(t, s, u.String())

	assert.Nil(t, u.UnmarshalText(nil))
	assert.True(t, u.IsNull())

	v, err := MustFromString(s).Value()

	assert.Equal(t, s, v)
	assert.Nil(t, err)

	v, err = NullUUID{}.Value()

	assert.Nil(t, v)
	assert.Nil(t, err)

	assert.Nil(t, u.Scan(s))
	assert.Equal(t, s, u.String())

	assert.Nil(t, u.Scan(""))
	assert.True(t, u.IsNull())

	assert.Nil(t, u.Scan([]byte(s)))
	assert.Equal(t, s, u.String())

	assert.Nil(t, u.Scan([]byte{}))
	assert.True(t, u.IsNull())

	assert.Nil(t, u.Scan(nil))
	assert.True(t, u.IsNull())

	err = u.Scan(12)
	assert.NotNil(t, err)
}
