// Copyright 2016 Orus.io. All rights reserved.

package api

import (
	"github.com/m4rw3r/uuid"
)

// ProcessStatus is the status of a process
//go:generate enum-marshaler -type ProcessStatus -prefix Process -lower -tests
type ProcessStatus int

const (
	// ProcessInitial is before anything happens
	ProcessInitial ProcessStatus = iota
	// ProcessRunning is when the process is running (envelopes are still being
	// transfered)
	ProcessRunning
	// ProcessPaused is when the process is paused. No envelope is emitted until
	// it is resumed, but incoming envelopes are still accepted.
	ProcessPaused
	// ProcessDone is when all transfers are finished and the process will
	// no longer move
	ProcessDone
	// ProcessError is when one or several errors occured. No more envelopes will
	// be accepted or emitted.
	ProcessError
)

const (
	// ControlProcessPrefix prefix for control process subjects
	ControlProcessPrefix = ControlPrefix + "process."
)

const (
	// ControlProcessQuery api to list process
	ControlProcessQuery = ControlProcessPrefix + ActionQuery
)

// Process is for exchanging informations on a process
type Process struct {
	ID                uuid.UUID
	TriggerEmitterID  uuid.UUID
	TriggerEnvelopeID uuid.UUID
	GraphID           uuid.UUID
	OldStatus         ProcessStatus
	Status            ProcessStatus
	State             string
}

// ControlProcessQueryReq is the request for process query
type ControlProcessQueryReq struct {
	IncludeTerminated bool // If true, includes terminated processes
}

// ControlProcessQueryRep is the response for process query
type ControlProcessQueryRep struct {
	Response
	ProcessList []Process
}

const (
	// ControlProcessControl api to pause/resume/cancel a process
	ControlProcessControl = ControlProcessPrefix + "control"
)

// ControlProcessControlReq is the request for process control
type ControlProcessControlReq struct {
	ProcessID uuid.UUID
	Command   string // One of 'pause', 'resume', 'cancel'
}

// ControlProcessControlRep is the response for process control
type ControlProcessControlRep struct {
	Response
	Process Process
}
