// Copyright 2016 Orus.io. All rights reserved.

package api

import (
	"github.com/m4rw3r/uuid"
)

// Registration is the subject for registration requests
const Registration = XbusAPIPrefix + "registration"

// RegistrationReq is a registration request
type RegistrationReq struct {
	Name      string      `json:"name,omitempty"`
	Type      AccountType `json:"type,omitempty"`
	CSR       string      `json:"csr,omitempty"`    // A Certificate Signing Request
	ActorList []Actor     `json:"actors,omitempty"` // A list of Actor to declare
}

// RegistrationStatus is the status of a registration
//go:generate enum-marshaler -type RegistrationStatus -prefix Registration -lower -tests
type RegistrationStatus int

const (
	// RegistrationPending is when the registration is expecting an
	// administrator to accept it
	RegistrationPending RegistrationStatus = iota
	// RegistrationAccepted is when the registration was accepted
	RegistrationAccepted
	// RegistrationRejected is when the registration was refused
	RegistrationRejected
)

// RegistrationRep is the reply from the registration service
type RegistrationRep struct {
	Response

	RegistrationStatus RegistrationStatus `json:"registration_status,omitempty"`
	ID                 uuid.UUID          `json:"id,omitempty"`
	SignedCertificate  string             `json:"signed_certificate,omitempty"`
	ServerCA           string             `json:"server_ca,omitempty"`
	ActorList          []Actor            `json:"actors,omitempty"`
}
